#include <iostream>
#include <fstream>
#include <iomanip>
#include <map>
#include <string>
#include <sstream>
#include <cctype>
#include <cerrno>

#ifdef GMP
#include <gmpxx.h>
#endif

using namespace std;

#include "SimpleChecker.h"

int main(int argc, char *argv[])
{
  if (argc!=2)
  {
    cout << "usage: " << argv[0] << " <OPB file>" << endl
	 << "\treads an interpretation on stdin and checks "
	 << "that the opb file is satisfied" << endl
	 << "\tif <OPB file> is a single dash (-), try to read "
	 << "the file name from stdin" << endl;

    exit(1);
  }

  try
  {
    string instance=argv[1];

    SimpleChecker checker;

    // reading the interpretation on stdin

    string s;

    while(cin.good())
    {
      getline(cin,s);

      if (!cin.good() && s.length()==0)
	break;

      // 
      string keyword="c solving ";
      if(instance=="-" && s.substr(0,keyword.length())==keyword)
      {
	int start,end;
	start=keyword.length();
	while(start<s.length() && isspace(s[start]))
	  ++start;

	end=s.length()-1;
	while(end>=start && isspace(s[end]))
	  --end;

	instance=s.substr(start,start-end+1);
      }

      // we're only concerned by a "v " line
      if (s.length()>=2 && s[0]=='v' && s[1]==' ')
      {
	istringstream f(s);
	string word;
	bool positive;

	f >> word; // skip "v "

	while (f.good())
	{
	  // read a literal
	  word="";
	  f >> word;
	  if (word=="")
	    break;

	  if (word[0]=='-')
	  {
	    positive=false;
	    word.erase(0,1);
	  }
	  else
	    positive=true;

	  checker.setValue(word,positive);
	}
      }
    }

    if (checker.nbAssignedVariables()==0)
    {
      cerr << "ERROR: no interpretation found !" << endl;
      exit(7);
    }

    checker.parse(instance.c_str());

    cout << "OK\t" << checker.getObjectiveValue() << endl;
    exit(0);
  }
  catch (exception &e)
  {
    cout.flush();
    cerr << "\n\tUnexpected exception :\n";
    cerr << "\t" << e.what() << endl;
    exit(1);
  }
}

